#!/bin/sh
DEFAULT_IP="192.168.10.100"
JBDATASIZE_CMD="./jbdatasize"
JBCOMP_CMD="./jbcomp"
JBFLASH_CMD="./jbflash"
SVF2JSVF_CMD="./svf2jsvf"

XSVF_EXT="\.xsvf"
JSVF_EXT="\.jsvf"
COMPED_EXT="\.jbc"

GetFileSize(){
    Size=`wc -c $1 | cut -f 1 -d ' '`
    echo $Size
    return $Size
}

#Option detect
while [ $1 ] ;
do
    if [ $1 = "-i" ] ; then
	shift
	IP_Address=$1
    fi
    if [ $1 = "-f" ] ; then
	shift
	Conf_File=$1
    fi
    if [ $1 = "-l" ] ; then
	TE7720_Mode="y"
    fi
    if [ $1 = "-L" ] ; then
	TE7720_Mode="n"
    fi
    if [ $1 = "-h" ] ; then
	printf " -i [IP_ADDRESS] : IP address\n"
	printf " -f [CONF_FILE]  : Configuration file\n"
	printf " -l              : TE7720 mode\n"
	printf " -L              : Not TE7720 mode\n"
	printf " -h              : this message\n"
	exit
    fi
    shift
done

#IP address setting
if ! [ $IP_Address ] ; then
    printf "IP ADDRESS [$DEFAULT_IP] : "
    read IP_Address

    if [ -z $IP_Address ] ; then
	IP_Address=$DEFAULT_IP
    fi
fi

#Configuration-file setting
if ! [ $Conf_File ] ; then
    printf "Configuration File : "
    read Conf_File
fi

if [ -z $Conf_File ] ; then
    printf "*** Please Input Configuration File!\n"
    exit
else
    if ! [ -e $Conf_File ] ; then
	printf "*** File not found : $Conf_File\n"
	exit
    else
	if [ -d $Conf_File ] ; then
	    printf "*** Input file is Directory!\n"
	    exit
	fi
    fi
fi

#TE7720_Mode setting
if ! [ $TE7720_Mode ] ; then
    printf "TE7720 mode ? (y/n) [n] : "
    read TE7720_Mode

    if [ -z $TE7720_Mode ] ; then
	TE7720_Mode="n"
    fi
fi

for i in y yes Y YES
do
    if [ $TE7720_Mode = $i ] ; then
	Option="-l"
	break
    fi
done

#SVF to JSVF format convert
if [ -z `echo $Conf_File | grep "\.xsvf"` ] && 
    [ -z `echo $Conf_File | grep "\.jsvf"` ] ; then
    $SVF2JSVF_CMD $Conf_File -o sample.jsvf
    if [ $? -ne 0 ] ; then
	exit
    fi
    Conf_File="sample.jsvf"
fi

#Get flash-size
Max_Size=`$JBDATASIZE_CMD $IP_Address`
if [ $? -ne 0 ] ; then
    exit
fi

#Get file-size
File_Size=`GetFileSize "$Conf_File"`

if [ $File_Size -le $Max_Size ] ; then
    Write_File=$Conf_File
else
    #need comspress configration-file
    if [ `echo $Conf_File | grep "\.xsvf"` ] ; then
	Comped_File=`echo $Conf_File | sed "s/$XSVF_EXT/$COMPED_EXT/"`
    else
	Comped_File=`echo $Conf_File | sed "s/$JSVF_EXT/$COMPED_EXT/"`
    fi

    $JBCOMP_CMD $Conf_File -o $Comped_File
    if [ $? -ne 0 ] ; then
	exit
    fi

    File_Size=`GetFileSize $Comped_File`
    if [ $File_Size -le $Max_Size ] ; then
	Option="$Option -c"
	Write_File=$Comped_File
    else
	printf "*** Input file size is too large!\n"
	rm $Comped_File
	exit
    fi
fi

echo "$JBFLASH_CMD $Option $IP_Address $Write_File"
$JBFLASH_CMD $Option $IP_Address $Write_File
if [ $Comped_File ] ; then
  rm $Comped_File
fi

