/*********************************************************************
 *
 * file name : discoverd.c
 * summary   : program to reply received armadillo-J discover packet
 * coded by  : mori
 * copyright : Atmark Techno
 * date(init): 2003.12.04
 * 
 ********************************************************************/

#include <iostream>
#include <cstdio>
#include <string.h>
#include <cstdlib>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <net/route.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <errno.h>

#include "discover_lib.h"

/* MACRO define */
#define REPLY_TIMEOUT	2	// 5s
#define TO_INADDR(x)	(*(struct in_addr*)&x)
#define OUTPUT_FORMAT	"%02x:%02x:%02x:%02x:%02x:%02x|DHCP %-3s|IP %-15s|MASK %-15s|GW %-15s"

using namespace std;

/**********************************************************************
 * summary : Output network info to find
 * return  :
 *********************************************************************/
void PrintNetworkInfo(const discover_packet_s* packet)
{
	char buff[512];
	char ip_address[16];
	char netmask[16];
	char gateway[16];

	strcpy(ip_address, inet_ntoa(TO_INADDR(packet->net_info.ip_address)));
	strcpy(netmask, inet_ntoa(TO_INADDR(packet->net_info.netmask)));
	strcpy(gateway, inet_ntoa(TO_INADDR(packet->net_info.gateway)));

	::sprintf(
			buff,
			OUTPUT_FORMAT,
			(unsigned char)packet->mac_addr[0],
			(unsigned char)packet->mac_addr[1],
			(unsigned char)packet->mac_addr[2],
			(unsigned char)packet->mac_addr[3],
			(unsigned char)packet->mac_addr[4],
			(unsigned char)packet->mac_addr[5],
			packet->net_info.use_dhcp? "yes" : "no",
			ip_address,
			netmask,
			gateway
			);
	cout << buff << endl;
}

/**********************************************************************
 * summary : main routine
 * return  : success -> 0, failed -> !0
 *********************************************************************/
int main(int argc, char **argv)
{
	CDiscover discover;

	if(!discover.Setup()){
		cerr << "failed to prepare socket" << endl;
		return -1;
	}
		
	LOG("sending find packet\n");
	unsigned int transactionID  = discover.SendFindPacket();
	if(!transactionID){
		cerr << "failed to send fine packet" << endl;
		return -1;
	}

	while(1){
		discover_packet_s packet;
		LOG("wait for reply\n");
		if(discover.RecvReplyPacket(&packet, transactionID, REPLY_TIMEOUT)){
			cout << "packet received" << endl;
			if(packet.message_type == ntohl(DISCOVER_REP)){
				PrintNetworkInfo(&packet);
			}
		}
		else{
			break;
		}
	}

	return 0;
}

