#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "jblib.h"

void usage()
{
    cout << "Usage: jbfiledata [IP ADDRESS] [OPTION]" << endl;
    cout << "Show file information in JTAG-Blazer" << endl;
    cout << " -h,\tshow this message" << endl;
}

int main(int argc, char** argv)
{
    int opt;
   	while((opt = getopt(argc, argv, "h")) != -1){
		switch(opt){
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}

	if(optind >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}

	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
		cerr << argv[0] << ": Invalid IP address" << endl;
		usage();
		return -1;
	}

	FlashFileData fileData;
	CJBLib lib(addr.s_addr);
	CJBLib::ErrorCode errCode = lib.FlashFileDataRequest(&fileData);

	if(errCode == CJBLib::JTAG_NO_ERROR){
		cout << "File Name   : " << fileData.fileName << endl;
		cout << "File Size   : " << fileData.fileSize << endl;
		cout << "Record Time : " << ctime(&fileData.recordTime);
		cout << "Compression : ";
		if(fileData.compressed)
			cout << "ON" << endl;
		else
			cout << "OFF" << endl;

		cout << "Option      : ";
		if(fileData.littleBear)
			cout << "TE7720 mode" << endl;
		else
			cout << "none" << endl;
	}
	else
		fprintf(stderr, "Failed (code : %08x)\n", (int)errCode);

	return (int)errCode;
}
