//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_main.c
//+ SUMMARY : SVFtoJSVF main function
//+     REV : 1.00 (2005/01/21) 
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

//+<< INCLUDE >>+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#include <stdio.h>
#ifdef WIN32
#include <io.h>
#else //WIN32
#include <unistd.h>
#endif //WIN32
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "stj_main.h"
#include "stj_common.h"
#include "stj_data_cache.h"
#include "stj_parse.h"
#include "stj_process.h"

//#define __FORCE_KEEP_JSVF_FILE
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : SVFtoJSVF
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int SVFtoJSVF(const char *svf_file,const char *jsvf_file){
  int SvfFd=0;
  int RetVal;

#ifdef WIN32
  SvfFd=open(svf_file,O_RDONLY | O_BINARY);
#else //WIN32
  SvfFd=open(svf_file,O_RDONLY);
#endif //WIN33
  if(SvfFd==-1){
    LOG_ERR("Not file open : %s\n",svf_file);
    return -1;
  }

  DataCacheInit(SvfFd);

  if(CreateJSVF(jsvf_file)==-1){
    LOG_ERR("Failed create file : %s\n",jsvf_file);
    close(SvfFd);
    return -1;
  }

  if((RetVal=SvfParse())==-1){
    LOG_ERR("[SVFtoJSVF] : error\n");
    ReleaseJSVF();
#ifndef __FORCE_KEEP_JSVF_FILE
    remove(jsvf_file);
#endif //__FORCE_KEEP_JSVF_FILE
  }else{
    ReleaseJSVF();
  }

  close(SvfFd);
  return RetVal;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : Option parameter
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#ifndef WIN32
typedef struct{
  char *str;
  int id;
}OptionInfo;
typedef enum _OptionID{
  OPTION_HELP=1,
  OPTION_OUTPUT_FILE,
}OptionID;
#define OPTION_COUNT 4
OptionInfo Option[OPTION_COUNT]={
  {"-h",OPTION_HELP},
  {"--help",OPTION_HELP},
  {"-o",OPTION_OUTPUT_FILE},
  {"--output-jsvf",OPTION_OUTPUT_FILE},
};
#endif //WIN32
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : Usage
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#ifndef WIN32
void Usage(void){
  printf("Usage : svf2jsvf [option] [svf_file]\n");
  printf("SVF to JSVF format converter\n");
  printf("option :\n");
  printf("  -h, --help        : this message.\n");
  printf("  -o, --output-jsvf : output file.(default is \"sample.jsvf\")\n");
}
#endif //WIN32
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : Usage
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#ifndef WIN32
void OptionError(char *option){
  printf("unrecognized option : `%s'\n",option);
  printf("Try `svf2jsvf --help' for more information.\n");
}
#endif //WIN32
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : main
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int main(int argc,char **argv){
#ifdef WIN32
  char file[256];
  printf("input file : ");
  scanf("%s",file);
  SVFtoJSVF(file,"sample.jsvf");
#else //WIN32
  char *svf_file=0;
  char *jsvf_file="sample.jsvf";
  int i,j;

  if(argc==1){
    Usage();
    return 0;
  }
  for(i=1;i<argc;i++){
    for(j=0;j<OPTION_COUNT;j++){
      if(strcmp(argv[i],Option[j].str)==0){
	switch(Option[j].id){
	case OPTION_HELP:
	  Usage();
	  return 0;
	case OPTION_OUTPUT_FILE:
	  if(argc-1==i){
	    OptionError(argv[i]);
	    return -1;
	  }
	  jsvf_file=argv[i+1];
	  i++;j=OPTION_COUNT;
	  break;
	default:
	  break;
	}
      }
    }
    if(j!=OPTION_COUNT+1){
      if(svf_file==0){
	svf_file=argv[i];
      }else{
	OptionError(argv[i]);
	return -1;
      }
    }
  }
  if(svf_file==0){
    Usage();
    return 0;
  }
  return SVFtoJSVF(svf_file,jsvf_file);
#endif //WIN32
  return 0;
}





