//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_process.c
//+ SUMMARY : SVFtoJSVF SVF process & JSVF process function
//+     REV : 1.00 (2005/01/21) 
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

//+<< INCLUDE >>+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <io.h>
#else //WIN32
#include <unistd.h>
#endif //WIN32

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "stj_common.h"
#include "stj_data_cache.h"
#include "stj_parse.h"
#include "stj_process.h"
#include "stj_shift.h"

#ifdef WIN32
#define inline __inline
#endif

static SVF_Trailer TrailerIR={0,0,0,0,0},TrailerDR={0,0,0,0,0};
static SVF_Header  HeaderIR ={0,0,0,0,0},HeaderDR ={0,0,0,0,0};

static SVF_Trailer PRE_Trailer={0,0,0,0,0};
static SVF_Header  PRE_Header ={0,0,0,0,0};

static int PRE_TDI_Len;
static int PRE_MASK_Len;
static int PRE_SMASK_Len;

static unsigned char PRE_TDI[TD_BUF_SIZE];
static unsigned char PRE_MASK[TD_BUF_SIZE];
static unsigned char PRE_SMASK[TD_BUF_SIZE];

static int PRE_SDRSIZE;

static int PRE_TDOMASK_Len;
static unsigned char PRE_TDOMASK[TD_BUF_SIZE];

static int RUNTEST_WaitMode;

#define Bit2ByteLen(bitlen) ((bitlen>>3)+((bitlen&0x00000007)?1:0))
#define Bit2CharLen(bitlen) ((bitlen>>2)+((bitlen&0x00000003)?1:0))

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DummyProcess
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void DummyProcess(void){
  char ReadData;
  while(1){
    if(DataCacheReadByte(&ReadData)==-1){
      return;
    }
    if(ReadData==';') return;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline int CompPreLen(const int dst_size,const int src_size){
  if(dst_size==src_size) return 0;
  return -1;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void SavePreBuf(unsigned char *dst,const unsigned char *src,
		       int *dst_size,const int src_size){
  int ByteLen=Bit2ByteLen(src_size);
  memcpy(dst,src,ByteLen);
  *dst_size=src_size;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void LoadPreBuf(unsigned char *dst,const unsigned char *src,
		       const int src_size){
  int ByteLen=Bit2ByteLen(src_size);
  memcpy(dst,src,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int CompPreBuf(unsigned char *dst,const unsigned char *src,
		      const int dst_size,const int src_size){
  int ByteLen;
  if(dst_size==src_size){
    ByteLen=Bit2ByteLen(src_size);
    if(memcmp(dst,src,ByteLen)==0){
      return 0;
    }
  }
  return -1;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int _JsvfFD=0;
int CreateJSVF(const char *file_name){
#ifdef WIN32
  _JsvfFD=open(file_name,O_CREAT|O_WRONLY|O_TRUNC|O_BINARY,0666);
#else //WIN32
  _JsvfFD=open(file_name,O_CREAT|O_WRONLY|O_TRUNC,0666);
#endif //WIN32
  if(_JsvfFD==-1){
    return -1;
  }
  SvfProcessInitialize(32);
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void ReleaseJSVF(void){
  SvfProcessEnd();
  close(_JsvfFD);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void WriteJSVF(const char *buf,const int size){
  write(_JsvfFD,buf,size);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfREPEAT(const unsigned char repeat){
  unsigned char WriteBuf[2]={JSVF_REPEAT,0};
  WriteBuf[1]=repeat;
  WriteJSVF(WriteBuf,2);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfENDIR(const unsigned char state){
  unsigned char WriteBuf[2]={JSVF_ENDIR,0};
  WriteBuf[1]=state;
  WriteJSVF(WriteBuf,2);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfENDDR(const unsigned char state){
  unsigned char WriteBuf[2]={JSVF_ENDDR,0};
  WriteBuf[1]=state;
  WriteJSVF(WriteBuf,2);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void JsvfSTATE(const unsigned char state){
  unsigned char WriteBuf[2]={JSVF_STATE,0};
  WriteBuf[1]=state;
  WriteJSVF(WriteBuf,2);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfRUNTEST(const int count){
  unsigned char WriteBuf[5]={JSVF_RUNTEST,0,0,0,0};
  int i;
  for(i=0;i<4;i++){
    WriteBuf[1+i]=(count>>((3-i)*8))&0x000000ff;
  }
  WriteJSVF(WriteBuf,5);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfWAIT(const unsigned char wait_state,
		     const unsigned char end_state,
		     const int count){
  unsigned char WriteBuf[7]={JSVF_WAIT,0,0,0,0,0,0};
  int i;
  WriteBuf[1]=wait_state;
  WriteBuf[2]=end_state;
  for(i=0;i<4;i++){
    WriteBuf[3+i]=(count>>((3-i)*8))&0x000000ff;
  }
  WriteJSVF(WriteBuf,7);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRSIZE(const int size){
  int i;
  unsigned char WriteBuf[5]={JSVF_SDRSIZE,0,0,0,0};
  if(PRE_SDRSIZE!=size){
    PRE_SDRSIZE=size;
    for(i=0;i<4;i++){
      WriteBuf[1+i]=(size>>((3-i)*8)) & 0x000000ff;
    }
    WriteJSVF(WriteBuf,5);
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfTDOMASK(const char *mask,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_TDOMASK};

  if(CompPreBuf(PRE_TDOMASK,mask,PRE_TDOMASK_Len,size)==0){
    return;
  }

  WriteJSVF(WriteBuf,1);
  WriteJSVF(mask,ByteLen);
  SavePreBuf(PRE_TDOMASK,mask,&PRE_TDOMASK_Len,size);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDR(const char *tdi,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDR};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRB(const char *tdi,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRB};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRC(const char *tdi,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRC};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRE(const char *tdi,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRE};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRTDO(const char *tdi,const char *tdo,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRTDO};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
  WriteJSVF(tdo,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRTDOB(const char *tdi,const char *tdo,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRTDOB};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
  WriteJSVF(tdo,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRTDOC(const char *tdi,const char *tdo,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRTDOC};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
  WriteJSVF(tdo,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSDRTDOE(const char *tdi,const char *tdo,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[1]={JSVF_SDRTDOE};

  WriteJSVF(WriteBuf,1);
  WriteJSVF(tdi,ByteLen);
  WriteJSVF(tdo,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfSIR(const char *tdi,const int size){
  int ByteLen=Bit2ByteLen(size);
  unsigned char WriteBuf[2]={JSVF_SIR,0};

  WriteBuf[1]=(unsigned char)size;

  WriteJSVF(WriteBuf,2);
  WriteJSVF(tdi,ByteLen);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void JsvfCOMPLETE(void){
  unsigned char WriteBuf[1]={JSVF_COMPLETE};
  WriteJSVF(WriteBuf,1);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#define FreeAllocatedBuf(buf) { \
  if(buf){ \
    free(buf); \
    buf=0; \
  } \
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void _FreeTrailerAndHeaderMember(SVF_Trailer_Header *th){
  FreeAllocatedBuf(th->TDI);
  FreeAllocatedBuf(th->TDO);
  FreeAllocatedBuf(th->MASK);
  FreeAllocatedBuf(th->SMASK);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void FreeTrailerMember(SVF_Trailer *tr){
  _FreeTrailerAndHeaderMember(tr);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void FreeHeaderMember(SVF_Header *hr){
  _FreeTrailerAndHeaderMember(hr);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessInitialize(const unsigned char repeat){
  RUNTEST_WaitMode=AnalyzeWaitMode();

  PRE_TDI_Len=0;
  PRE_MASK_Len=0;
  PRE_SMASK_Len=0;
  PRE_SDRSIZE=0;
  PRE_TDOMASK_Len=0;

  JsvfREPEAT(repeat);
  JsvfRUNTEST(0);
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessEnd(void){
  JsvfCOMPLETE();

  FreeTrailerMember(&TrailerIR);
  FreeTrailerMember(&TrailerDR);
  FreeHeaderMember(&HeaderIR);
  FreeHeaderMember(&HeaderDR);

  FreeTrailerMember(&PRE_Trailer);
  FreeHeaderMember(&PRE_Header); 
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void InitInfoRUNTEST(InfoRUNTEST *info){
  info->WaitState=0xff;
  info->EndState=0xff;
  info->Wait=0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : ScanRuntestInfo
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void ScanRuntestInfo(InfoRUNTEST *info){
  static int DefaultEndState=STATE_IDLE;

  int WaitState=-1;
  int EndState=-1;

  int ParseFlag=0;

  char Label[256];
  int Count=0;
  int Flag=0;

  while(1){
    memset(Label,0,256);
    if((Flag=GetLabel(Label))==-1){
      ;
      break;
    }
    
    if(memcmp(Label,"RESET",5)==0){
      if(ParseFlag!=2){
	WaitState=STATE_RESET;
	ParseFlag=1;
      }else if(ParseFlag==2){
	EndState=STATE_RESET;
	ParseFlag=3;
      }
      goto LAST_LABEL_CHECK;      
    }else if(memcmp(Label,"IDLE",4)==0){
      if(ParseFlag!=2){
	WaitState=STATE_IDLE;
	ParseFlag=1;
      }else if(ParseFlag==2){
	EndState=STATE_IDLE;
	ParseFlag=3;
      }
      goto LAST_LABEL_CHECK;
    }else if(memcmp(Label,"DRPAUSE",7)==0){
      if(ParseFlag!=2){
	WaitState=STATE_PAUSE_DR;
	ParseFlag=1;
      }else if(ParseFlag==2){
	EndState=STATE_PAUSE_DR;
	ParseFlag=3;
      }
      goto LAST_LABEL_CHECK;
    }else if(memcmp(Label,"IRPAUSE",7)==0){
      if(ParseFlag!=2){
	WaitState=STATE_PAUSE_IR;
	ParseFlag=1;
      }else if(ParseFlag==2){
	EndState=STATE_PAUSE_IR;
	ParseFlag=3;
      }
      goto LAST_LABEL_CHECK;
    }

    if(memcmp(Label,"ENDSTATE",8)==0){
      ParseFlag=2;
      goto LAST_LABEL_CHECK;  
    }

    if(memcmp(Label,"TCK",3)==0){
      goto LAST_LABEL_CHECK;
    }else if(memcmp(Label,"SCK",3)==0){
      goto LAST_LABEL_CHECK;
    }

    if(Count==0){
      if((Count=(int)strtol(Label,0,10))==0){
	LOG_ERR("[SvfProcessRUNTEST] Unknown Label : %s\n",Label);
	break;
      }
    }
  LAST_LABEL_CHECK:
    if(Flag==1){
      break;
    }
  }
  if(Flag==0){
    DummyProcess();
  }

  if(WaitState!=-1){
    if(EndState==-1){
      DefaultEndState=WaitState;
      EndState=DefaultEndState;
    }
  }
  info->WaitState=(unsigned char)WaitState;
  info->EndState=(unsigned char)EndState;
  info->Wait=Count;
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : SvfProcessRUNTEST
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#define __WAIT_ENABLE
void SvfProcessRUNTEST(void){
  InfoRUNTEST Info;

  InitInfoRUNTEST(&Info);

  ScanRuntestInfo(&Info);
  
  if(Info.WaitState || RUNTEST_WaitMode){
    if(Info.WaitState==0xff) Info.WaitState=STATE_IDLE;
    if(Info.EndState==0xff) Info.EndState=STATE_IDLE;
#ifdef __WAIT_ENABLE
    JsvfWAIT(Info.WaitState,Info.EndState,Info.Wait);
#else
    JsvfSTATE(Info.WaitState);
    JsvfRUNTEST(Info.Wait);
    JsvfSTATE(Info.EndState);
#endif
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessTRST(void){
  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessPIO(void){
  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessPIOMAP(void){
  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessFREQUENCY(void){
  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessENDIR(void){
  char Label[256];
  int Flag=0;

  memset(Label,0,256);
  if((Flag=GetLabel(Label))==-1){
    return;
  }

  if(memcmp(Label,"IDLE",4)==0){
    JsvfENDIR(ENDIR_IDLE);
  }else if(memcmp(Label,"IRPAUSE",7)==0){
    JsvfENDIR(ENDIR_PAUSE_IR);
  }else if(memcmp(Label,"RESET",5)==0){
    LOG_MSG("[SvfProcessENDIR] NO SUPPORT LABEL : %s\n",Label);
  }else{
    LOG_MSG("[SvfProcessENDIR] NO SUPPORT LABEL : %s\n",Label);
  }

  if(Flag==0) DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessENDDR(void){
  char Label[256];
  int Flag=0;

  memset(Label,0,256);
  if((Flag=GetLabel(Label))==-1){
    return;
  }

  if(memcmp(Label,"IDLE",4)==0){
    JsvfENDDR(ENDDR_IDLE);
  }else if(memcmp(Label,"DRPAUSE",7)==0){
    JsvfENDDR(ENDDR_PAUSE_DR);
  }else if(memcmp(Label,"RESET",5)==0){
    LOG_ERR("[SvfProcessENDDR] NO SUPPORT LABEL : %s\n",Label);
  }else{
    LOG_ERR("[SvfProcessENDDR] NO SUPPORT LABEL : %s\n",Label);
  }

  if(Flag==0) DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessSTATE(void){
  char Label[256];
  int Flag=0;

  while(1){
    memset(Label,0,256);
    if((Flag=GetLabel(Label))==-1){
      break;
    }

    if(memcmp(Label,"RESET",5)==0){
      JsvfSTATE(STATE_RESET);
    }else if(memcmp(Label,"IDLE",4)==0){
      JsvfSTATE(STATE_IDLE);
    }else if(memcmp(Label,"DRSELECT",8)==0){
      JsvfSTATE(STATE_SELECT_DR);
    }else if(memcmp(Label,"DRCAPTURE",9)==0){
      JsvfSTATE(STATE_CAPTURE_DR);
    }else if(memcmp(Label,"DRSHIFT",7)==0){
      JsvfSTATE(STATE_SHIFT_DR);
    }else if(memcmp(Label,"DREXIT1",7)==0){
      JsvfSTATE(STATE_EXIT1_DR);
    }else if(memcmp(Label,"DRPAUSE",7)==0){
      JsvfSTATE(STATE_PAUSE_DR);
    }else if(memcmp(Label,"DREXIT2",7)==0){
      JsvfSTATE(STATE_EXIT2_DR);
    }else if(memcmp(Label,"DRUPDATE",8)==0){
      JsvfSTATE(STATE_UPDATE_DR);
    }else if(memcmp(Label,"IRSELECT",8)==0){
      JsvfSTATE(STATE_SELECT_IR);
    }else if(memcmp(Label,"IRCAPTURE",9)==0){
      JsvfSTATE(STATE_CAPTURE_IR);
    }else if(memcmp(Label,"IRSHIFT",7)==0){
      JsvfSTATE(STATE_SHIFT_IR);
    }else if(memcmp(Label,"IREXIT1",7)==0){
      JsvfSTATE(STATE_EXIT1_IR);
    }else if(memcmp(Label,"IRPAUSE",7)==0){
      JsvfSTATE(STATE_PAUSE_IR);
    }else if(memcmp(Label,"IREXIT2",7)==0){
      JsvfSTATE(STATE_EXIT2_IR);
    }else if(memcmp(Label,"IRUPDATE",8)==0){
      JsvfSTATE(STATE_UPDATE_IR);
    }else{
      LOG_MSG("[SvfProcessSTATE] NO SUPPORT LABEL : %s\n",Label);
      return;
    }

    if(Flag==1) break;
  }

  if(Flag==0) DummyProcess();  
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int GetTestDataCount(void){
  unsigned char ReadData;
  int Count=0;
  while(1){
    DataCacheReadByte(&ReadData);
    if(ReadData==')') break;
    if(ReadData=='('){Count=0;continue;}
    if('0'<= ReadData && ReadData <= '9'){
      Count++;
    }
    if('a'<= ReadData && ReadData <= 'f'){
      Count++;
    }
    if('A'<= ReadData && ReadData <= 'F'){
      Count++;
    }
  }  
  return Count;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void ScanCommandInfo(CommandInfo *info){
  char Label[256];
  int Flag;

  DC_Position Cur_Pos;

  memset(Label,0,256);
  memset(&Cur_Pos,0,sizeof(Cur_Pos));

  DataCacheSavePosition(&Cur_Pos);

  if((Flag=GetLabel(Label))==-1){
    goto END;
  }
  if((info->BitLen=strtol(Label,0,10))==0){
    goto END;
  }

  while(1){
    if((Flag=GetLabel(Label))==-1){
      goto END;
    }

    if(memcmp(Label,"TDI",4)==0){
      info->TDI_Flag=1;
      info->TDI_CharLen=GetTestDataCount();
      DataCacheSavePosition(&info->TDI_Pos);
    }else if(memcmp(Label,"TDO",4)==0){
      info->TDO_Flag=1;
      info->TDO_CharLen=GetTestDataCount();
      DataCacheSavePosition(&info->TDO_Pos);
    }else if(memcmp(Label,"MASK",5)==0){
      info->MASK_Flag=1;
      info->MASK_CharLen=GetTestDataCount();
      DataCacheSavePosition(&info->MASK_Pos);
    }else if(memcmp(Label,"SMASK",6)==0){
      info->SMASK_Flag=1;
      info->SMASK_CharLen=GetTestDataCount();
      DataCacheSavePosition(&info->SMASK_Pos);
    }else{
      LOG_MSG("[ScanCommandInfo] NO SUPPORT LABEL : %s\n",Label);
    }

    if(Flag==1){
      goto END;
    }
  }
 END:
  DataCacheLoadPosition(&Cur_Pos);
  return;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//#define __DEBUG_SCANINFO
#ifdef __DEBUG_SCANINFO
static void ScanInfoOutput(const CommandInfo *info){
  LOG_CMSG(CL_GREEN,"BitLen : %d\n",info->BitLen);
  LOG_CMSG(CL_GREEN,"TDI    : %s\n",info->TDI_Flag ? "yes":"no");
  LOG_CMSG(CL_GREEN,"TDO    : %s\n",info->TDO_Flag ? "yes":"no");
  LOG_CMSG(CL_GREEN,"MASK   : %s\n",info->MASK_Flag ? "yes":"no");
  LOG_CMSG(CL_GREEN,"SMASK  : %s\n",info->SMASK_Flag ? "yes":"no");
}
#else //__DEBUG_SCANINFO
#define ScanInfoOutput(info)
#endif //__DEBUG_SCANINFO
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int GetShiftLen(int bitlen,int mode){
  switch(mode & MASK_MODE1){
  case MODE_IR:
    return HeaderIR.BitLen+TrailerIR.BitLen+bitlen;
  case MODE_DR:
    return HeaderDR.BitLen+TrailerDR.BitLen+bitlen; 
  default:
    return -1;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline SVF_Header *GetHeader(const int mode){
  if((mode & MASK_MODE1)==MODE_DR){
    return &HeaderDR;
  }else if((mode & MASK_MODE1)==MODE_IR){
    return &HeaderIR;
  }else{
    return 0;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline SVF_Trailer *GetTrailer(const int mode){
  if((mode & MASK_MODE1)==MODE_DR){
    return &TrailerDR;
  }else if((mode & MASK_MODE1)==MODE_IR){
    return &TrailerIR;
  }else{
    return 0;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int GetCharTestData(unsigned char *data){
  unsigned char ReadData;
  while(1){
    DataCacheReadBytePrevious(&ReadData);
    if('0'<= ReadData && ReadData <= '9'){
      break;
    }
    if('a'<= ReadData && ReadData <= 'f'){
      break;
    }
    if('A'<= ReadData && ReadData <= 'F'){
      break;
    }
  }
  *data=ReadData;
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : GetTestData
//+   NOTE : 
//+  PARAM : pos       : TestDatãt@C|WV
//+          buf       : i[obt@
//+          copy_size : i[obt@փRs[rbgTCY
//+          data_size : TestDatãrbgTCY
//+          offset    : TestDatǎ납̃ItZbg
//+          test_data_count : SVFt@CTestDatãf[^JEg
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void GetTestData(DC_Position *pos,
			unsigned char *buf,
			const int copy_size,const int data_size,
			const int offset,const int test_data_count){
  int i;
  unsigned char Data_1st[2]={0,0};
  unsigned char Data_2nd[TD_BUF_SIZE+1];
  int TestDataCount=test_data_count;
  int SrcOffset;
  int SrcSize;
  int CharLen=copy_size+(((offset)%4)?4-(offset%4):0);
  CharLen=Bit2CharLen(CharLen);

  memset(Data_2nd,0,TD_BUF_SIZE+1);

  //ۂ̃eXgf[^֏C
  if(TestDataCount<CharLen){
    CharLen=TestDataCount;
  }

  DataCacheLoadPositionPrevious(pos);

  //f[^擾PUi֕ϊ
  for(i=CharLen-1;i>=0;i--){
    if(GetCharTestData(&Data_1st[0])==-1){
      LOG_ERR("GetTestData:error\n");
      return;
    }
    if(CharLen%2==0){
      Data_2nd[i/2] |= (unsigned char)
	(strtol(Data_1st,0,16)<<(4*((i%2)?0:1)));
    }else{
      Data_2nd[(i+1)/2] |= (unsigned char)
	(strtol(Data_1st,0,16)<<(4*((i%2)?1:0)));
    }
  }

  //rbgVtg
  SrcSize=(CharLen/2+CharLen%2)*8;
  if(data_size==copy_size){
    SrcOffset=0;
  }else{
    SrcOffset=((offset%4-copy_size%4)+4)%4;
  }
  RequestData(buf,Data_2nd,copy_size,SrcSize,0,SrcOffset,0,0);

  //|WVۑ
  if(offset%4==0){
    DataCacheSavePosition(pos);
  }else{
    char dummy;
    DataCacheReadByte(&dummy);
    DataCacheSavePosition(pos);
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//#define __DEBUG_PREPARE_TEST_DATA
static void GenerateData(CommandInfo *info,
			 unsigned char *buf,
			 const int bitlen,const int count,const int mode){
  SVF_Trailer *Trailer =GetTrailer(mode);
  SVF_Header  *Header  =GetHeader(mode);

  unsigned char tmp_buf[TD_BUF_SIZE];
  int RetVal;
  int WishSize;
  int WishOffset;
  int BufBitPtr=0;

  memset(tmp_buf,0,TD_BUF_SIZE);

  if(count<Header->BitLen){
    //need header
    WishOffset=0;
    if(bitlen<Header->BitLen){
      WishSize=0;
    }else{
      if(info->BitLen < bitlen-Header->BitLen){
	WishSize=info->BitLen;
      }else{
	WishSize=bitlen-Header->BitLen;
      }
    }
  }else{
    //need trailer
    WishSize=0;
    WishOffset=count-Header->BitLen;

    if(info->BitLen < count+bitlen){
      WishSize=info->BitLen-(count-Header->BitLen);
    }else{
      if(info->BitLen < count-Header->BitLen + bitlen){
	WishSize=info->BitLen-(count-Header->BitLen);
      }else{
	WishSize=bitlen;
      }
    }
  }

  WishOffset=info->BitLen-WishOffset-WishSize;
  WishOffset=info->BitLen-WishOffset;

  switch(mode & MASK_MODE2){
  case MODE_TDI:
    GetTestData(&info->TDI_Pos,tmp_buf,WishSize,info->BitLen,
		WishOffset,info->TDI_CharLen);
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("----TDI-------------------------------------------\n");
    LOG_MSG("After:GetTestData\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    if(Header->BitLen!=0){
      RetVal=RequestData(buf,Header->TDI,bitlen,Header->BitLen,
			 BufBitPtr,0,count,0);
      BufBitPtr+=RetVal;
    }
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:Header\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    RetVal=RequestData(buf,tmp_buf,bitlen,WishSize,
		       BufBitPtr,0,BufBitPtr,Header->BitLen);
    BufBitPtr+=RetVal;
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:TestData\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    if(Trailer->BitLen!=0){
      RetVal=RequestData(buf,Trailer->TDI,bitlen,Trailer->BitLen,
			 BufBitPtr,0,count,Header->BitLen+info->BitLen);
    }
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:Trailer\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
    getchar();
#endif //__DEBUG_PREPARE_TEST_DATA
    break;
  case MODE_TDO:
    GetTestData(&info->TDO_Pos,tmp_buf,WishSize,info->BitLen,
		WishOffset,info->TDO_CharLen);

    RetVal=RequestData(buf,Header->TDO,bitlen,Header->BitLen,
		       BufBitPtr,0,count,0);
    BufBitPtr+=RetVal;
    
    RetVal=RequestData(buf,tmp_buf,bitlen,WishSize,
		       BufBitPtr,0,BufBitPtr,Header->BitLen);
    BufBitPtr+=RetVal;
    
    RetVal=RequestData(buf,Trailer->TDO,bitlen,Trailer->BitLen,
		       BufBitPtr,0,count,Header->BitLen+info->BitLen);
    break;
  case MODE_MASK:
    GetTestData(&info->MASK_Pos,tmp_buf,WishSize,info->BitLen,
		WishOffset,info->MASK_CharLen);

    RetVal=RequestData(buf,Header->MASK,bitlen,Header->BitLen,
		       BufBitPtr,0,count,0);
    BufBitPtr+=RetVal;
    
    RetVal=RequestData(buf,tmp_buf,bitlen,WishSize,
		       BufBitPtr,0,BufBitPtr,Header->BitLen);
    BufBitPtr+=RetVal;
    
    RetVal=RequestData(buf,Trailer->MASK,bitlen,Trailer->BitLen,
		       BufBitPtr,0,count,Header->BitLen+info->BitLen);
    break;
  case MODE_SMASK:
    GetTestData(&info->SMASK_Pos,tmp_buf,WishSize,info->BitLen,
		WishOffset,info->SMASK_CharLen);
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("----SMASK-----------------------------------------\n");
    LOG_MSG("After:GetTestData\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    RetVal=RequestData(buf,Header->SMASK,bitlen,Header->BitLen,
		       BufBitPtr,0,count,0);
    BufBitPtr+=RetVal;
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:Header\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    RetVal=RequestData(buf,tmp_buf,bitlen,WishSize,
		       BufBitPtr,0,BufBitPtr,Header->BitLen);
    BufBitPtr+=RetVal;
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:TestData\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
#endif //__DEBUG_PREPARE_TEST_DATA
    RetVal=RequestData(buf,Trailer->SMASK,bitlen,Trailer->BitLen,
		       BufBitPtr,0,count,Header->BitLen+info->BitLen);
#ifdef __DEBUG_PREPARE_TEST_DATA
    LOG_MSG("After:Trailer\n");
    DebugPrintHex(tmp_buf,Bit2ByteLen(WishSize));
    getchar();
#endif //__DEBUG_PREPARE_TEST_DATA
    break;
  default:
    break;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#define __ENABLE_SDRTDO_BCE
void SvfProcessSDR(void){
  CommandInfo Info;
  int SDR_Size=0;
  int Count=0;
  int i;

  unsigned char TDI_Buf[TD_BUF_SIZE];
  unsigned char TDO_Buf[TD_BUF_SIZE];
  unsigned char MASK_Buf[TD_BUF_SIZE];
  unsigned char SMASK_Buf[TD_BUF_SIZE];

  memset(&Info,0,sizeof(Info));
  ScanCommandInfo(&Info);
  ScanInfoOutput(&Info);

  if((SDR_Size=GetShiftLen(Info.BitLen,MODE_DR))==-1){
    LOG_MSG("SvfProcessSDR:error\n");
    return;
  }

  if(SDR_Size>MAX_TD_LEN && Info.TDO_Flag==0){
    //XSDR(BCE)
    while(SDR_Size-Count){
      int CurByteSize;
      int CurSize;
      int CurMode;
      if(Count==0){
	//SDRB
	CurSize=MAX_TD_LEN;
	CurMode=1;
      }else if(SDR_Size-Count<=MAX_TD_LEN){
	//SDRE
	CurSize=SDR_Size-Count;
	CurMode=2;
      }else{
	//SDRC
	CurSize=MAX_TD_LEN;
	CurMode=3;
      }

      JsvfSDRSIZE(CurSize);

      CurByteSize=Bit2ByteLen(CurSize);
      memset(TDI_Buf,0,CurByteSize);
      memset(SMASK_Buf,0xff,CurByteSize);

      if(Info.TDI_Flag){
	GenerateData(&Info,TDI_Buf,CurSize,Count,MODE_DR | MODE_TDI);
	SavePreBuf(PRE_TDI,TDI_Buf,&PRE_TDI_Len,CurSize);
      }else{
	if(CompPreLen(PRE_TDI_Len,CurSize)==0){
	  LoadPreBuf(TDI_Buf,PRE_TDI,PRE_TDI_Len);
	}
      }

      if(Info.SMASK_Flag){
	GenerateData(&Info,SMASK_Buf,CurSize,Count,MODE_DR | MODE_SMASK);
	SavePreBuf(PRE_SMASK,SMASK_Buf,&PRE_SMASK_Len,CurSize);
      }else{
	if(CompPreLen(PRE_SMASK_Len,CurSize)==0){
	  LoadPreBuf(SMASK_Buf,PRE_SMASK,PRE_SMASK_Len);
	}
      }

      for(i=0;i<Bit2ByteLen(CurSize);i++){
	TDI_Buf[i]=(TDI_Buf[i] & SMASK_Buf[i]);
      }

      switch(CurMode){
      case 1:
	JsvfSDRB(TDI_Buf,CurSize);
	break;
      case 2:
	JsvfSDRE(TDI_Buf,CurSize);
	break;
      case  3:
	JsvfSDRC(TDI_Buf,CurSize);
	break;
      }
      Count+=CurSize;
    }
  }else{
    while(SDR_Size-Count){
      int CurByteSize;
      int CurSize;
#ifdef __ENABLE_SDRTDO_BCE
      int CurMode;
      if(Count==0){
	if(SDR_Size>=MAX_TD_LEN){
	  CurSize=MAX_TD_LEN;
	  CurMode=1;
	}else{
	  CurSize=SDR_Size;
	  CurMode=0;
	}
      }else if(SDR_Size-Count<=MAX_TD_LEN){
	CurSize=SDR_Size-Count;
	CurMode=2;
      }else{
	CurSize=MAX_TD_LEN;
	CurMode=3;
      }
#else //__ENABLE_SDRTDO_BCE
      if(Count==0){
	if(SDR_Size>=MAX_TD_LEN){
	  CurSize=MAX_TD_LEN;
	}else{
	  CurSize=SDR_Size;
	}
      }else if(SDR_Size-Count<=MAX_TD_LEN){
	CurSize=SDR_Size-Count;
      }else{
	CurSize=MAX_TD_LEN;
      }
#endif //__ENABLE_SDRTDO_BCE

      //XSDRTDO
      JsvfSDRSIZE(CurSize);

      CurByteSize=Bit2ByteLen(CurSize);
      memset(TDI_Buf,0,CurByteSize);
      memset(TDO_Buf,0,CurByteSize);
      memset(MASK_Buf,0,CurByteSize);
      memset(SMASK_Buf,0xff,CurByteSize);

      if(Info.TDI_Flag){
	GenerateData(&Info,TDI_Buf,CurSize,Count,MODE_DR | MODE_TDI);
	SavePreBuf(PRE_TDI,TDI_Buf,&PRE_TDI_Len,CurSize);
      }else{
	if(CompPreLen(PRE_TDI_Len,CurSize)==0){
	  LoadPreBuf(TDI_Buf,PRE_TDI,PRE_TDI_Len);
	}
      }
      
      if(Info.SMASK_Flag){
	GenerateData(&Info,SMASK_Buf,CurSize,Count,MODE_DR | MODE_SMASK);
	SavePreBuf(PRE_SMASK,SMASK_Buf,&PRE_SMASK_Len,CurSize);
      }else{
	if(CompPreLen(PRE_SMASK_Len,CurSize)==0){
	  LoadPreBuf(SMASK_Buf,PRE_SMASK,PRE_SMASK_Len);
	}
      }
      
      if(Info.MASK_Flag){
	GenerateData(&Info,MASK_Buf,CurSize,Count,MODE_DR | MODE_MASK);
	JsvfTDOMASK(MASK_Buf,CurSize);
	SavePreBuf(PRE_MASK,MASK_Buf,&PRE_MASK_Len,CurSize);
      }else{
	if(Info.TDO_Flag){
	  if(CompPreLen(PRE_MASK_Len,CurSize)==0){
	    LoadPreBuf(MASK_Buf,PRE_MASK,PRE_MASK_Len);
	  }
	}
	JsvfTDOMASK(MASK_Buf,CurSize);
      }
      
      if(Info.TDO_Flag){
	GenerateData(&Info,TDO_Buf,CurSize,Count,MODE_DR | MODE_TDO);
      }
      
      for(i=0;i<Bit2ByteLen(CurSize);i++){
	TDI_Buf[i]=(TDI_Buf[i] & SMASK_Buf[i]);
      }

#ifdef __ENABLE_SDRTDO_BCE
      switch(CurMode){
      case 0:
	JsvfSDRTDO(TDI_Buf,TDO_Buf,CurSize);
	break;
      case 1:
	JsvfSDRTDOB(TDI_Buf,TDO_Buf,CurSize);
	break;
      case 2:
	JsvfSDRTDOE(TDI_Buf,TDO_Buf,CurSize);
	break;
      case  3:
	JsvfSDRTDOC(TDI_Buf,TDO_Buf,CurSize);
	break;
      }
#else //__ENABLE_SDRTDO_BCE
      JsvfSDRTDO(TDI_Buf,TDO_Buf,CurSize);
#endif //__ENABLE_SDRTDO_BCE
      Count+=CurSize;
    }
  }
  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessSIR(void){
  CommandInfo Info;
  int SIR_Size=0;

  int i;
  unsigned char TDI_Buf[TD_BUF_SIZE];
  unsigned char MASK_Buf[TD_BUF_SIZE];
  unsigned char SMASK_Buf[TD_BUF_SIZE];
  int SIR_ByteSize;

  memset(&Info,0,sizeof(Info));
  ScanCommandInfo(&Info);
  ScanInfoOutput(&Info);

  if((SIR_Size=GetShiftLen(Info.BitLen,MODE_IR))==-1){
    LOG_MSG("SvfProcessSIR:error\n");
    return;
  }

  SIR_ByteSize=Bit2ByteLen(SIR_Size);

  memset(TDI_Buf,0,SIR_ByteSize);
  memset(MASK_Buf,0,SIR_ByteSize);
  memset(SMASK_Buf,0xff,SIR_ByteSize);

  if(Info.TDI_Flag){
    GenerateData(&Info,TDI_Buf,SIR_Size,0,MODE_IR | MODE_TDI);
    SavePreBuf(PRE_TDI,TDI_Buf,&PRE_TDI_Len,SIR_Size);
  }else{
    if(CompPreLen(PRE_TDI_Len,SIR_Size)==0){
      LoadPreBuf(TDI_Buf,PRE_TDI,PRE_TDI_Len);
    }
  }

  if(Info.SMASK_Flag){
    GenerateData(&Info,SMASK_Buf,SIR_Size,0,MODE_IR | MODE_SMASK);
    SavePreBuf(PRE_SMASK,SMASK_Buf,&PRE_SMASK_Len,SIR_Size);
  }else{
    if(CompPreLen(PRE_SMASK_Len,SIR_Size)==0){
      LoadPreBuf(SMASK_Buf,PRE_SMASK,PRE_SMASK_Len);
    }
  }
  
  if(Info.MASK_Flag){
    GenerateData(&Info,MASK_Buf,SIR_Size,0,MODE_IR | MODE_MASK);
    SavePreBuf(PRE_MASK,MASK_Buf,&PRE_MASK_Len,SIR_Size);
  }
    
  for(i=0;i<Bit2ByteLen(SIR_Size);i++){
    TDI_Buf[i]=(TDI_Buf[i] & SMASK_Buf[i]);
  }

  JsvfSIR(TDI_Buf,SIR_Size);

  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//#define __debug_trailer_header
#ifdef __debug_trailer_header
static void TrailerHeaderOutput(const char *msg,const SVF_Trailer_Header *th){
  LOG_MSG("[[%s]]\n",msg);
  LOG_MSG("BitLen : %d\n",th->BitLen);
  LOG_MSG("TDI::\n");
  DebugPrintHex(th->TDI,Bit2ByteLen(th->BitLen));  
  LOG_MSG("TDO::\n");
  DebugPrintHex(th->TDO,Bit2ByteLen(th->BitLen));  
  LOG_MSG("MASK::\n");
  DebugPrintHex(th->MASK,Bit2ByteLen(th->BitLen));  
  LOG_MSG("SMASK::\n");
  DebugPrintHex(th->SMASK,Bit2ByteLen(th->BitLen));  
}
#endif //__debug_trailer_header
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
inline void _SvfProcessTH(SVF_Trailer_Header *th,
			  SVF_Trailer_Header *pre){
  CommandInfo Info;

  memset(&Info,0,sizeof(Info));
  ScanCommandInfo(&Info);
  ScanInfoOutput(&Info);

  _FreeTrailerAndHeaderMember(th);
  th->BitLen=0;

  if(Info.BitLen==0){
    _FreeTrailerAndHeaderMember(pre);
    pre->BitLen=0;
    DummyProcess();
    return;
  }
  th->BitLen=Info.BitLen;

  th->TDI=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
  th->TDO=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
  th->MASK=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
  th->SMASK=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));

  if(Info.TDI_Flag){
    GetTestData(&Info.TDI_Pos,th->TDI,th->BitLen,th->BitLen,
		(th->BitLen/4+((th->BitLen%4)?1:0)) *4,Info.TDI_CharLen);
  }
  if(Info.TDO_Flag){
    GetTestData(&Info.TDO_Pos,th->TDO,th->BitLen,th->BitLen,
		(th->BitLen/4+((th->BitLen%4)?1:0)) *4,Info.TDO_CharLen);
  }
  if(Info.MASK_Flag){
    GetTestData(&Info.MASK_Pos,th->MASK,th->BitLen,th->BitLen,
		(th->BitLen/4+((th->BitLen%4)?1:0)) *4,Info.MASK_CharLen);
  }
  if(Info.SMASK_Flag){
    GetTestData(&Info.SMASK_Pos,th->SMASK,th->BitLen,th->BitLen,
		(th->BitLen/4+((th->BitLen%4)?1:0)) *4,Info.SMASK_CharLen);
  }

  if(th->BitLen==pre->BitLen){
    if(Info.TDI_Flag==0){
      memcpy(th->TDI,pre->TDI,Bit2ByteLen(pre->BitLen));
    }
    if(Info.TDO_Flag==0){
      memcpy(th->TDO,pre->TDO,Bit2ByteLen(pre->BitLen));
    }
    if(Info.MASK_Flag==0){
      memcpy(th->MASK,pre->MASK,Bit2ByteLen(pre->BitLen));
    }
    if(Info.SMASK_Flag==0){
      memcpy(th->SMASK,pre->SMASK,Bit2ByteLen(pre->BitLen));
    }
  }else{
    pre->BitLen=th->BitLen;
    pre->TDI=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
    pre->TDO=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
    pre->MASK=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
    pre->SMASK=(unsigned char *)calloc(1,Bit2ByteLen(Info.BitLen));
    
    memcpy(pre->TDI,th->TDI,Bit2ByteLen(th->BitLen));
    memcpy(pre->TDO,th->TDO,Bit2ByteLen(th->BitLen));
    memcpy(pre->MASK,th->MASK,Bit2ByteLen(th->BitLen));
    memcpy(pre->SMASK,th->SMASK,Bit2ByteLen(th->BitLen));
  }

  DummyProcess();
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessTIR(void){
  _SvfProcessTH(&TrailerIR,&PRE_Trailer);
#ifdef __debug_trailer_header
  TrailerHeaderOutput("--TIR--",&TrailerIR);
  getchar();
#endif //__debug_trailer_header
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessTDR(void){
  _SvfProcessTH(&TrailerDR,&PRE_Trailer);
#ifdef __debug_trailer_header
  TrailerHeaderOutput("--TDR--",&TrailerDR);
  getchar();
#endif //__debug_trailer_header
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessHIR(void){
  _SvfProcessTH(&HeaderIR,&PRE_Header);
#ifdef __debug_trailer_header
  TrailerHeaderOutput("--HIR--",&HeaderIR);
  getchar();
#endif //__debug_trailer_header
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void SvfProcessHDR(void){
  _SvfProcessTH(&HeaderDR,&PRE_Header);
#ifdef __debug_trailer_header
  TrailerHeaderOutput("--HDR--",&HeaderDR);
  getchar();
#endif //__debug_trailer_header
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : RuleProcessRUNTEST
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 0 : Not found RUNTEST,1 : Found RUNTEST,-1 : Error
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int RuleProcessRUNTEST(DC_Position *next,InfoRUNTEST *info,int flag){
  static int ContinueFlag=0;
  int ComID;
  int Count=0;
  DC_Position Pos;

  if(RUNTEST_WaitMode){
    return 0;
  }
  memset(&Pos,0,sizeof(Pos));
  
  if(flag==1){
    DataCacheLoadPosition(next);
  }

  DataCacheSavePosition(&Pos);
  
  if(flag!=0){

    if((ComID=SeekCommand())<=0){
      DataCacheLoadPosition(&Pos);
      return -1;
    }
    switch(ComID){
    case SVF_COM_SIR:
      ContinueFlag++;
      break;
    case SVF_COM_SDR:
      ContinueFlag++;
      break;
    default:
      break;
    }
  }
    
  DummyProcess();

  while(1){
    if((ComID=SeekCommand())<=0){
      if(Count==0){
	DataCacheLoadPosition(&Pos);
	if(flag){
	  JsvfRUNTEST(0);
	}
	return -1;
      }else{
	break;
      }
    }

    switch(ComID){
    case SVF_COM_RUNTEST:

      if(flag){
        ContinueFlag=0;
	JsvfRUNTEST(0);
      }
      ScanRuntestInfo(info);
      if(info->WaitState!=0xff){
	break;
      }

      Count+=info->Wait;
      DataCacheSavePosition(next);
      continue;
    default:
      break;
    }
    break;
  }

  DataCacheLoadPosition(&Pos);
  if(Count!=0){
    if(ContinueFlag>=2){
      JsvfRUNTEST(0);
      ContinueFlag=0;
    }  
    JsvfRUNTEST(Count);    
    return 1;
  }
  if(ContinueFlag!=0){
    if(ContinueFlag>=2){
      JsvfRUNTEST(0);
      ContinueFlag=0;
      return 0;
    }
    return 2;
  }
  return 0;
}

